function [alphaf,pif,tf,zf,logLmax]=cluster_categ(pl,plz,r,visu,nessai)

% E2M algorithm for the latent class model with uncertain attributes and/or  
% class membership (mixture model with categorical attributes)

% INPUT
%
% pl: (n,p,R) matrix of attribute value plausibilities; pl(i,j,h) is the
%      plausibility that attribute j takes value h for object i
% plz: (n,g) matrix of class plausibilities; plz(i,k) is the plausbility
%      that object i belongs to class k
% r: vector of length p; r(j) is the number of modalities of attribute j 
% visu : 1 to display intermediate results, 0 otherwise
% nessai: number of trials with random initial parameter values 
%
% OUTPUT
%
% alphaf: (g,p,R) matrix of estimated values of parameter alpha;
%         alphaf(k,j,h) is the estimated probability that attribute j takes 
%         value h, conditionally on class k
% pif: (1,g) vector of estimated prior class probabilities
% tf: (n,g) matrix of posterior probabilities; tf(i,k) is the estimated 
%     probability that object i belongs to class k
% zf: (n,1) vector of estimated classes; zf(i) is the estimated class for 
%     object i 
% logLmax: best final value of log-likelihood
%
% References:
%
% T. Denoeux. Maximum likelihood from evidential data: an extension of the 
% EM algorithm. In C. Borgelt et al. (Eds), Combining soft computing and 
% statistical methods in data analysis (Proceedings of SMPS 2010, Oviedo, Spain, 
% September 28 - October 1, 2010), Advances in Intelligent and Soft Computing, 
% pages 181-188, Springer, 2010.
%
% T. Denoeux, Maximum likelihood estimation from Uncertain Data in the Belief
% Function Framework, IEEE Transactions on Knowledge and Data Engineering
% (to appear), 2011.
%
% Written by Thierry Denoeux, 11 Augut 2011.

n=size(pl,1);
p=size(pl,2);
g=size(plz,2);
R=max(r);

alpha=zeros(g,p,R);

logLmax=-inf;

for essai=1:nessai,

    for k=1:g,
        for j=1:p,
            x=rand(1,r(j)-1);
            pp=diff([0 sort(x) 1]);
            alpha(k,j,1:r(j))=pp;
        end,
    end;

    x=rand(1,g-1);
    pi=diff([0 sort(x) 1]);


    go_on=1;
    K=0;

    gamma=zeros(g,p,n,R);
    t=zeros(n,g);
    while go_on,
        K=K+1;
        for k=1:g,
            x=ones(n,1);
            for j=1:p
                y=repmat(squeeze(alpha(k,j,1:r(j)))',n,1).*squeeze(pl(:,j,1:r(j)));
                x=x.*sum(y,2);
                gamma(k,j,:,1:r(j))=y./repmat(sum(y,2),1,r(j));
            end;
            t(:,k)=x.*repmat(pi(k),n,1).*plz(:,k);
        end;
        LL=sum(t,2);
        t=t./repmat(LL,1,g);


        logL(K)=sum(log(LL));
        if isnan(logL(K)), go_on=0; logL(K)=-inf; end;
        if visu, disp([essai K logL(K)]); end;

        if (K>1) & ((logL(K)-logL(K-1))./abs(logL(K-1)) < 1e-6),
            go_on=0;
        end
        % M step

        for k=1:g,
            for j=1:p,
                alpha(k,j,1:r(j))=sum(squeeze(gamma(k,j,:,1:r(j))).*repmat(t(:,k),1,r(j)));
                alpha(k,j,1:r(j))=alpha(k,j,1:r(j))./sum(alpha(k,j,1:r(j)),3);
            end,
        end;
        pi=mean(t);
    end;

    if logL(K)>logLmax,
        alphaf=alpha;
        pif=pi;
        tf=t;
        [tmp,zf]=max(tf,[],2);
        logLmax=logL(K);
    end;

end;






