function [muf,sig2f,pif,tf,zf,logLmax]=cluster_norm(m,s2,plz,visu,nessai)


% E2M algorithm for the Gaussian micture model with uncertain attributes 
% and/or class membership  
% Covariance matrices in each class are assumed to be diagonal 

% INPUT
%
% m: (n,p) matrix of mean attribute values; m(i,j) is the
%      mean of the Gaussian pdf describing uncertainty on attribute j for object i
% s2: (n,p) matrix of mean attribute variances; s2(i,j) is the
%      variance of the Gaussian pdf describing uncertainty on attribute j for object i
% plz: (n,g) matrix of class plausibilities; plz(i,k) is the plausbility
%      that object i belongs to class k 
% visu : 1 to display intermediate results, 0 otherwise
% nessai: number of trials with random initial parameter values 
%
% OUTPUT
%
% muf: (g,p) matrix of estimated values of parameter mu; muf(k,j) is the 
%      estimated mean of attribute j conditionally on class k
% sig2f: (g,p) matrix of estimated values of parameter sigma^; sig2f(k,j) is the 
%      estimated variance of attribute j conditionally on class k
% pif: (1,g) vector of estimated prior class probabilities
% tf: (n,g) matrix of posterior probabilities; tf(i,k) is the estimated 
%     probability that object i belongs to class k
% zf: (n,1) vector of estimated classes; zf(i) is the estimated class for 
%     object i 
% logLmax: best final value of log-likelihood
%
% References:
%
% T. Denoeux. Maximum likelihood from evidential data: an extension of the 
% EM algorithm. In C. Borgelt et al. (Eds), Combining soft computing and 
% statistical methods in data analysis (Proceedings of SMPS 2010, Oviedo, Spain, 
% September 28 - October 1, 2010), Advances in Intelligent and Soft Computing, 
% pages 181-188, Springer, 2010.
%
% T. Denoeux, Maximum likelihood estimation from Uncertain Data in the Belief
% Function Framework, IEEE Transactions on Knowledge and Data Engineering
% (to appear), 2011.
%
% Written by Thierry Denoeux, 11 Augut 2011.



[n,p]=size(m);
g=size(plz,2);


logLmax=-inf;

for essai=1:nessai,

   mu=m(randsample(n,g),:);
   sig2=repmat(var(m),g,1);

    x=rand(1,g-1);
    pi=diff([0 sort(x) 1]);


    go_on=1;
    K=0;

    muijk=zeros(n,p,g);
    s2ijk=muijk;
    phi=muijk;
    t=zeros(n,g);
    while go_on,
        K=K+1;
        for k=1:g,
            x=ones(n,1);
            for j=1:p
                D=sig2(k,j)+s2(:,j);
                muijk(:,j,k)=(mu(k,j)*s2(:,j)+m(:,j)*sig2(k,j))./D;
                s2ijk(:,j,k)=(sig2(k,j)*s2(:,j))./D;
                phi(:,j,k)=normpdf(m(:,j),mu(k,j),sqrt(D));
                x=x.*phi(:,j,k);
            end;
         %   keyboard
            t(:,k)=x.*repmat(pi(k),n,1).*plz(:,k);
        end;
        LL=sum(t,2);
        t=t./repmat(LL,1,g);


        logL(K)=sum(log(LL));
        if visu, disp([essai K logL(K)]); end;

        if (K>1) & (abs(logL(K)-logL(K-1))./abs(logL(K-1)) < 1e-6),
            go_on=0;
        end
        % M step

        for k=1:g,
            C=sum(t(:,k));
            mu(k,:)=sum(squeeze(muijk(:,:,k)).*repmat(t(:,k),1,p))./C;
            sig2(k,:)=sum((squeeze(muijk(:,:,k).^2+s2ijk(:,:,k)) ...
                - 2 * repmat(mu(k,:),n,1).*squeeze(muijk(:,:,k))...
                + repmat(mu(k,:).^2,n,1)).*repmat(t(:,k),1,p))./C;
        end;
        pi=mean(t);
    end;

    if logL(K)>logLmax,
        muf=mu;
        sig2f=sig2;
        pif=pi;
        tf=t;
        [tmp,zf]=max(tf,[],2);
        logLmax=logL(K);
    end;

end;






